-----------------------------------------------------------------------------------------------
-- Client Lua Script for SimpleSpellPower
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Apollo"
require "GameLib"
require "Window"
require "Unit"

-------------------------------------------------------------------------------------
-- SimpleSpellPower Module Definition
-----------------------------------------------------------------------------------------------
local SimpleSpellPower = {} 
local run = true

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SimpleSpellPower:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function SimpleSpellPower:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {"ClassResources"}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
-----------------------------------------------------------------------------------------------
-- SimpleSpellPower OnLoad
-----------------------------------------------------------------------------------------------
function SimpleSpellPower:OnLoad()
	self:InitializeHooks()	
end

function SimpleSpellPower:InitializeHooks()
	local crb = Apollo.GetAddon("ClassResources")
	crb.OnSlingerUpdateTimer = self.OnSlingerUpdateTimer
	crb.OnSlingerEnteredCombat = self.OnSlingerEnteredCombat
end


function SimpleSpellPower:OnSlingerUpdateTimer()
	local unitPlayer = GameLib.GetPlayerUnit()
	local nResourceMax = unitPlayer:GetMaxResource(4)
	local nResourceCurrent = unitPlayer:GetResource(4)
	local nResourceMaxDiv4 = nResourceMax / 4
	local bSurgeActive = GameLib.IsSpellSurgeActive()
	local bInCombat = unitPlayer:IsInCombat()
	
	if run then
		self.wndMain:FindChild("SlingerBaseFrame_OutOfCombat"):Show(false)
		self.wndMain:FindChild("SlingerBaseFrame_InCombat"):Show(false)
	end	
		

	-- Mana
	local nManaMax = math.floor(unitPlayer:GetMaxMana())
	local nManaCurrent = math.floor(unitPlayer:GetMana())
	self.wndMain:FindChild("ManaProgressBar"):SetMax(nManaMax)
	self.wndMain:FindChild("ManaProgressBar"):SetProgress(nManaCurrent)
	self.wndMain:FindChild("ManaProgressBar"):SetTooltip(String_GetWeaselString(Apollo.GetString("SpellslingerResource_FocusTooltip"), nManaCurrent, nManaMax))
	self.wndMain:FindChild("ManaProgressBar"):SetStyleEx("EdgeGlow", bInCombat and (nManaCurrent / nManaMax < 0.97))
	self.wndMain:FindChild("ManaProgressText"):SetText(nManaCurrent == nManaMax and "" or (math.floor(nManaCurrent / nManaMax * 100).."%"))
	self.wndMain:FindChild("ManaProgressText"):SetTextColor(bInCombat and ApolloColor.new("ffffc757") or ApolloColor.new("UI_TextHoloTitle"))
	self.wndMain:FindChild("ManaProgressBacker"):Show(nManaCurrent ~= nManaMax)

	-- Nodes
	local strNodeTooltip = String_GetWeaselString(Apollo.GetString("Spellslinger_SpellSurge"), nResourceCurrent, nResourceMax)
	for idx, wndCurr in pairs({ self.wndSlinger1, self.wndSlinger2, self.wndSlinger3, self.wndSlinger4 }) do
		local nPartialProgress = nResourceCurrent - (nResourceMaxDiv4 * (idx - 1)) -- e.g. 250, 500, 750, 1000
		local bThisBubbleFilled = nPartialProgress >= nResourceMaxDiv4
		wndCurr:FindChild("NodeProgress"):SetMax(nResourceMaxDiv4)
		wndCurr:FindChild("NodeProgress"):SetProgress(nPartialProgress, 100)

		if not bSurgeActive then
			wndCurr:FindChild("NodeFilled"):SetSprite("CM_SpellslingerSprites:sprSlinger_Node_"..idx.."Disabled")
			wndCurr:FindChild("NodeProgress"):SetFullSprite("CM_SpellslingerSprites:sprSlinger_NodeBar_OutOfCombat")
			self.wndMain:FindChild("LargeSurgeSigil"):SetSprite("CM_SpellslingerSprites:sprSlinger_LargeSigil_OutOfCombat")
			self.wndMain:FindChild("ManaProgressBar"):SetFullSprite("CM_SpellslingerSprites:sprSlinger_ManaBar_OutOfCombat")
		end	
		if bSurgeActive then
			wndCurr:FindChild("NodeFilled"):SetSprite("CM_SpellslingerSprites:sprSlinger_Node_"..idx.."Normal")
			wndCurr:FindChild("NodeProgress"):SetFullSprite("CM_SpellslingerSprites:sprSlinger_NodeBar_InCombatOrange")
			self.wndMain:FindChild("LargeSurgeSigil"):SetSprite("CM_SpellslingerSprites:sprSlinger_LargeSigil_InCombat")
			self.wndMain:FindChild("ManaProgressBar"):SetFullSprite("CM_SpellslingerSprites:sprSlinger_ManaBar_InCombat")
		end	
		if not bThisBubbleFilled then
			wndCurr:FindChild("NodeProgress"):SetFullSprite("CM_SpellslingerSprites:sprSlinger_NodeBar_InCombatRed")
		end
		wndCurr:FindChild("NodeFilled"):Show(not bInCombat or bThisBubbleFilled, false, 0.2)
		wndCurr:FindChild("NodeRed"):Show(bInCombat and not bThisBubbleFilled, false, 0.2)

		-- Check last state
		local nLast = wndCurr:GetData() or nPartialProgress
		if bInCombat and nLast ~= nResourceMaxDiv4 and nPartialProgress == nResourceMaxDiv4 then -- Wasn't filled, now filled = just filled flash
			wndCurr:FindChild("NodeFlash"):SetSprite("CM_SpellslingerSprites:sprSlinger_NodeBar_Flash_Orange")
		end
		wndCurr:SetData(nPartialProgress)
		wndCurr:SetTooltip(strNodeTooltip)
	end

	-- Surge
	self.wndMain:FindChild("LargeSurgeSigil"):Show(bSurgeActive, bSurgeActive, 0.4)
	self.wndMain:FindChild("LargeSurgeGlow"):Show(bSurgeActive and bInCombat, bSurgeActive, 0.4)

	self.wndMain:FindChild("SlingerBaseFrame_InCombat"):SetSprite(nResourceCurrent < nResourceMaxDiv4 and "sprSlinger_Base_InCombatRed" or "sprSlinger_Base_InCombatOrange")

	self:HelperToggleVisibiltyPreferences(self.wndMain, unitPlayer)
end

function SimpleSpellPower:OnSlingerEnteredCombat(unitPlayer, bInCombat)
	if unitPlayer ~= GameLib.GetPlayerUnit() or not self.wndMain or not self.wndMain:IsValid() then
		return
	end
end

-----------------------------------------------------------------------------------------------
-- SimpleSpellPower Instance
-----------------------------------------------------------------------------------------------
local SimpleSpellPowerInst = SimpleSpellPower:new()
SimpleSpellPowerInst:Init()
